@@ -45,7 +45,14 @@ $lDebug    = isset($_ENV['MYENV_DEBUG']) ? $_ENV['MYENV_DEBUG'] : '';
 // $lConfFile = $lBaseDir . '/' . 'etc/' . 'myenv.conf';
 $lConfFile = '/etc/myenv/myenv.conf';
 
-$aConfiguration = getConfiguration($lConfFile);
+list($ret, $aConfiguration) = getConfiguration($lConfFile);
+
+if ( count($aConfiguration) == 0 ) {
+	$rc = 229;
+	$msg = "Configuration $lConfFile does not exist or is not readable (rc=$rc).";
+  error($msg);
+  exit($rc);
+}
 
 if ( ! array_key_exists($lInstance, $aConfiguration) ) {
   $rc = 244;
