@@ -159,7 +159,7 @@ else {
 }
 
 // Get configuration and display for selection
-$aConfiguration = getConfiguration($lConfFile);
+list($ret, $aConfiguration) = getConfiguration($lConfFile);
 
 // Loop as long as customer wants some changes
 do {
