@@ -11,8 +11,12 @@ PHP=`which php`
 ret=$?
 
 if [ $ret -ne 0 ] ; then
-  rc=1
-  echo "ERROR: Cannot find PHP."
+	rc=1
+	echo "ERROR: Cannot find PHP."
+	echo "Please install PHP as follows:"
+	echo "RedHat/CentOS: shell> sudo yum install php-cli";
+	echo "Ubuntu/Debian: shell> sudo apt-get install php5-cli";
+	echo "SLES/OpenSuSE: shell> sudo zypper install php5";
   exit $rc
 fi
 
