@@ -3,6 +3,9 @@
 
 $rc = 0;
 
+// todo: should not be hard coded but configurable!
+ini_set('date.timezone', 'Europe/Zurich');
+
 // Caution $_ENV variables are not be known here yet when variables_order
 // does NOT contain E!!!
 if ( isset($_ENV['MYENV_BASE']) ) {
@@ -50,8 +53,11 @@ switch ( $argv[1] ) {
 case 'start':
   fputs($handle, date("Y-m-d H:i:s") . " Starting $MyBasename\n");
   // find all databases
-  $databases = printStartingDatabases();
-  if ( $databases == '' ) {
+  list($ret, $databases) = printStartingDatabases();
+  if ( $ret != OK ) {
+		exit($ret);
+  }
+  if ( count($databases) == 0 ) {
     fputs($handle, date("Y-m-d H:i:s") . " No databases to start.\n");
   }
   else {
@@ -72,8 +78,11 @@ case 'stop':
   fputs($handle, date("Y-m-d H:i:s") . " Stopping $MyBasename\n");
   // find all started databases and stop them (better to stop all started db,
   // that are still running or even all running dbs?)
-  $databases = printStartingDatabases();
-  if ( $databases == '' ) {
+  list($ret, $databases) = printStartingDatabases();
+  if ( $ret != OK ) {
+		exit($ret);
+  }
+  if ( count($databases) == 0 ) {
     fputs($handle, date("Y-m-d H:i:s") . " No databases to start.\n");
   }
   else {
