@@ -74,7 +74,8 @@ fi
 # Synced means node is ready again
 if [ "$STATUS" == "Synced" ] ; then
 
-  if [ `rule_exists` -eq 1 ] ; then
+  # Can have more than 1 row matching
+  if [ `rule_exists` -gt 1 ] ; then
   	for src in $LB_IP ; do
 			cmd="sudo /sbin/iptables --delete INPUT --source=$src --protocol tcp --dport $MYSQL_PORT -j REJECT"
 			echo $cmd >>$LOG
@@ -88,7 +89,7 @@ else
   # which is possibly a bit overkill. So we do not use the --check but
   # a grep as suggested otherwise...
   # cmd="sudo /sbin/iptables --check INPUT --protocol tcp --dport $MYSQL_PORT -j REJECT"
-  if [ `rule_exists` -ne 1 ] ; then
+  if [ `rule_exists` -lt 1 ] ; then
   	for src in $LB_IP ; do
 			cmd="sudo /sbin/iptables --insert INPUT --source=$src --protocol tcp --dport $MYSQL_PORT -j REJECT"
 			echo $cmd >>$LOG
