@@ -4,6 +4,7 @@
 $basedir = dirname(dirname(__FILE__));
 include($basedir . '/lib/Constants.inc');
 include($basedir . '/lib/mysql_bman.inc');
+include($basedir . '/lib/mysql_bman_catalog.inc');
 include($basedir . '/lib/myEnv.inc');
 
 $rc = OK;
@@ -12,10 +13,10 @@ ini_set('date.timezone', 'Europe/Zurich');
 
 // This variable is only for test automatization
 $gTesting       = false;
-$gaBackupPolicy = array('daily', 'weekly', 'monthly', 'quarterly', 'yearly');
+$gaBackupPolicy = array('daily', 'weekly', 'monthly', 'quarterly', 'yearly', 'binlog');
 $gLogFile       = './mysql_bman.log';;
 $gBackupDir     = './bck';
-
+$gRelease       = '1.1.0';
 
 // Check requirements
 
@@ -56,6 +57,9 @@ $longopts  = array(
 , 'backupdir:'
 , 'cleanup'
 , 'config:'
+, 'catalog:'
+, 'catalog-name:'
+, 'create'
 , 'help'
 , 'log:'
 , 'instance-name:'
@@ -69,6 +73,7 @@ $longopts  = array(
 , 'simulate'
 , 'target:'
 , 'type:'
+, 'upgrade'
 );
 
 // The parsing of options will end at the first non-option found, any-
@@ -127,8 +132,8 @@ $aMyCnf = array();
 foreach ( $aMyCnfFiles as $filename ) {
 
 	if ( is_readable($filename) ) {
-		print "Reading configuation from $filename\n";
-		$aConfig = @parse_ini_file($filename, TRUE);
+		print "\nReading configuation from $filename\n";
+		list($ret, $aConfig) = parseConfigFile($filename);
 		// An error on parsing!
 		if ( $aConfig === false ) {
 			$rc = 164;
@@ -156,7 +161,7 @@ foreach ( $aMyCnfFiles as $filename ) {
 		}
 	}
 	else {
-		// print "No file $filename found.\n";
+		// print "No config file $filename found.\n";
 	}
 }
 
@@ -170,6 +175,9 @@ $gMysqlBman = '';
 if ( isset($aParameter['config']) && is_readable($aParameter['config']) ) {
   $gMysqlBman = $aParameter['config'];
   list($rc, $aConfig) = readConfigurationFile($gMysqlBman);
+  if ( $rc != OK ) {
+		exit($rc);
+	}
 	foreach ( $aConfig as $key => $value ) {
 		$aOptions[$key] = $value;
 	}
@@ -191,6 +199,10 @@ if ( (! isset($aOptions['backupdir'])) || ($aOptions['backupdir'] == '') ) {
   $aOptions['backupdir'] = './bck';
 }
 
+if ( (! isset($aOptions['catalog-name'])) || ($aOptions['catalog-name'] == '') ) {
+	$aOptions['catalog-name'] = 'bman_catalog';
+}
+
 // Minimum requirements is a log file
 if ( ! isset($aOptions['log']) ) {
   // tee does not work here yet!
@@ -209,10 +221,20 @@ if ( $ret != OK ) {
   exit($rc);
 }
 
+// todo: Should be done above checkArguments!
 if ( ! array_key_exists('mode', $aOptions) ) {
   $aOptions['mode'] = 'logical';
 }
 
+// Set type and policy to '' if catalog operations are wanted to avoid
+// Undefined index: type error messages...
+if ( ! isset($aOptions['type']) && (array_key_exists('create', $aOptions) || array_key_exists('upgrade', $aOptions)) ) {
+	$aOptions['type'] = 'catalog';
+	$aOptions['policy'] = array_key_exists('create', $aOptions) ? 'create' : 'upgrade';
+	$aOptions['mode'] = '';
+}
+
+// todo: Should be done above checkArguments!
 // If type = schema and no schema specified add empty schema option
 if ( $aOptions['type'] == 'schema' ) {
   if ( ! isset($aOptions['schema']) ) {
@@ -220,6 +242,7 @@ if ( $aOptions['type'] == 'schema' ) {
   }
 }
 
+
 // $gCatalog = '';
 // $aAuxiliary = '';
 // Target is not needed when type is cleanup
@@ -229,7 +252,18 @@ if ( isset($aOptions['target']) ) {
   
   if ( $ret != OK ) {
     $rc = 109;
-    tee("  ERROR wrong connect string (rc=$rc)!");
+    tee("  ERROR wrong connect string in target (rc=$rc)!");
+    exit($rc);
+  }
+}
+
+$aCatalog = array();
+if ( isset($aOptions['catalog']) ) {
+  list($ret, $aCatalog) = parseConnectString($aOptions['catalog']);
+  
+  if ( $ret != OK ) {
+    $rc = 196;
+    tee("  ERROR wrong connect string in catalog(rc=$rc)!");
     exit($rc);
   }
 }
@@ -237,6 +271,7 @@ if ( isset($aOptions['target']) ) {
 
 // If user or password was set in .my.cnf update target
 
+// todo: Should be done above checkArguments!
 if ( isset($aOptions['user']) ) {
 	$aTarget['user'] = $aOptions['user'];
 }
@@ -246,8 +281,11 @@ if ( isset($aOptions['password']) ) {
 // Adjust target again
 
 if ( isset($aTarget['host']) ) {
-  print_r($aTarget);
-  $aOptions['target'] = $aTarget['user'] . ':' . '******' . '@' . $aTarget['host'] . ':' . $aTarget['port'];
+	$aOptions['target'] = $aTarget['user'] . ':' . '******' . '@' . $aTarget['host'] . ':' . $aTarget['port'];
+}
+
+if ( isset($aCatalog['host']) ) {
+	$aOptions['catalog'] = $aCatalog['user'] . ':' . '******' . '@' . $aCatalog['host'] . ':' . $aCatalog['port'];
 }
 
 
@@ -257,6 +295,8 @@ if ( $gMysqlDump == '' ) {
   exit($rc);
 }
 
+// todo: Should be done above checkArguments!
+// possibly becomes obsolete later when d/h is introduced
 if ( isset($aOptions['retention']) ) {
   $aOptions['retention'] = intval($aOptions['retention']);
 }
@@ -288,10 +328,31 @@ if ($gMysqlBman != '' ) {
   }
 }
 else {
-  tee("\nNo configuration file.\n");
+	tee("\nNo mysql_bman configuration file.\n");
 }
 
+
+// Prepare command line
+$cmdln = '';
+foreach ( $argv as $val ) {
+	if ( preg_match('/(--target=)(.*)(@.*)/', $val, $matches)
+	  || preg_match('/(--catalog=)(.*)(@.*)/', $val, $matches)
+	  ) {
+
+		$val = $matches[1];
+		if ( preg_match('|(.*[/:])(.*)|', $matches[2], $ms) ) {
+			$val .= $ms[1] . '******';
+		}
+		$val .= $matches[3];
+	}
+	$cmdln .= $val . ' ';
+}
+
+tee('Command line: ' . $cmdln . "\n");
+
+
 // Display parameter from command line options
+
 tee("Configuration from command line options");
 foreach ( $aParameter as $key => $value ) {
 	if ( ($key == 'target') || ($key == 'catalog') ) {
@@ -299,7 +360,7 @@ foreach ( $aParameter as $key => $value ) {
 		tee(sprintf("  %-13s = %s", $key, $aT['user'] . ':' . '******' . '@' . $aT['host'] . ':' . $aT['port']   ));
 		continue;
 	}
-  tee(sprintf("  %-13s = %s", $key, $value));
+	tee(sprintf("  %-13s = %s", $key, $value));
 }
 
 // Display resulting configuration
@@ -308,74 +369,210 @@ foreach ( $aOptions as $key => $value ) {
 	if ( $key == 'password' ) {
 		$value = '******';
 	}
-  tee(sprintf("  %-13s = %s", $key, $value));
+	tee(sprintf("  %-13s = %s", $key, $value));
 }
 
-tee("\nLogging to " . $gLogFile);
+list($ret, $aVersion) = getBmanVersion();
+
+tee("\nLogging to   " . $gLogFile);
 tee("Backupdir is " . $gBackupDir);
-tee("Hostname is " . $gHostname);
+tee("Hostname is  " . $gHostname);
+tee("Version is   " . $gRelease . ' (catalog v' . $aVersion['hr_version'] . ')');
 if ( array_key_exists('simulate', $aOptions) ) {
-  tee("This is a SIMULATION only!");
-}
-$gBackupTimestamp = date("Y-m-d_H-i-s");
-tee("\nStart " . $aOptions['type'] . ' ' . $aOptions['policy'] . " backup at $gBackupTimestamp");
-
-// Is empty for type cleanup
-$gTargetConfiguration = array();
-if ( (count($aTarget) > 0) && ($aOptions['type'] != 'cleanup') ) {
-  $gTargetConfiguration = getTargetConfiguration($aTarget);
+	tee("This is a SIMULATION only!");
 }
 
-if ( checkBackupStructure($gBackupDir) != 0 ) {
-  $rc = 129;
-  tee("  ERROR: Problems creating backup structure (rc=$rc).");
-  exit($rc);
+
+// We will use a catalog
+if ( array_key_exists('catalog', $aOptions) ) {
+
+	list($rc, $mysqli) = openCatalog($aCatalog, $aOptions['catalog-name']);
+	if ( $rc != OK ) {
+		$rc = 224;
+		$msg = "Connection to catalog failed (rc=$rc).";
+		tee("  ERROR: $msg");
+		exit($rc);
+	}
+
+	// On catalog creation
+	if ( array_key_exists('create', $aOptions) ) {
+		// noop
+	}
+	// Otherwise : upgrade or normal backup
+	else {
+
+		list($ret, $cnt) = checkIfCatalogExists($mysqli, $aOptions['catalog-name']);
+		if ( ($ret == OK) && ($cnt != 0) ) {
+			list($rc, $aCatalogVersion) = getCatalogVersion($mysqli, $aOptions['catalog-name']);
+			tee("Catalog is   " . $aOptions['catalog-name'] . " version " . $aCatalogVersion['hr_version']);
+		}
+		// Catalog does NOT exist
+		else {
+			$rc = 203;
+			$msg = "Catalog does NOT exist. Please create catalog first with --create (rc=$rc).";
+			tee("  ERROR: $msg");
+			exit($rc);
+		}
+
+		// Check if catalog version matches
+
+		list($ret, $aBmanVersion) = getBmanVersion();
+
+		if ( $aCatalogVersion['mr_version'] != $aBmanVersion['mr_version'] ) {
+			$rc = 199;
+			$msg = "Catalog has not the same version as software. (rc=$rc).";
+			tee(" ERROR: $msg");
+			exit($rc);
+		}
+	}
+}
+
+
+// Not necessary if it is cleanup or catalog
+
+if ( ($aOptions['type'] != 'catalog') && ($aOptions['type'] != 'cleanup') ) {
+
+	// Should be check and create Backup structure
+	$rc = checkBackupStructure($gBackupDir);
+	if ( $rc != OK ) {
+		$rc = 129;
+		tee("  ERROR: Problems creating backup structure (rc=$rc).");
+		exit($rc);
+	}
+}
+
+
+// todo: should go to optionCheck
+if ( array_key_exists('archive', $aOptions) ) {
+	if ( ! isset($aOptions['archivedir']) || ($aOptions['archivedir'] == '' ) ) {
+		$rc = 134;
+		tee("  ERROR: Archivedir is not specified (rc=$rc).");
+		exit($rc);
+	}
 }
 
 if ( array_key_exists('archive', $aOptions) ) {
-  if ( ! isset($aOptions['archivedir']) || ($aOptions['archivedir'] == '' ) ) {
-    $rc = 134;
-    tee("  ERROR: Archivedir is not specified (rc=$rc).");
-    exit($rc);
-  }
+	$rc = checkArchiveStructure($aOptions['archivedir']);
+	if ( $rc != OK ) {
+		$rc = 147;
+		tee("  ERROR: Problems creating archive structure (rc=$rc).");
+		exit($rc);
+	}
 }
 
-if ( array_key_exists('archive', $aOptions) && (checkArchiveStructure($aOptions['archivedir']) != 0) ) {
-  $rc = 147;
-  tee("  ERROR: Problems creating archive structure (rc=$rc).");
-  exit($rc);
+
+// Is empty for type cleanup
+
+$gTargetConfiguration = array();
+if ( (count($aTarget) > 0) && ($aOptions['type'] != 'cleanup') ) {
+	list($rc, $gTargetConfiguration) = getTargetConfiguration($aTarget);
+	if ( $rc != OK ) {
+		exit($rc);
+	}
 }
 
+
+// Start here:
+
+$gBackupTimestamp = date("Y-m-d_H-i-s");
+
+// Write backup start tag into catalog
+if ( array_key_exists('catalog', $aOptions) && (! array_key_exists('create', $aOptions)) && (! array_key_exists('upgrade', $aOptions)) ) {
+
+  $aBackupInformation = array(
+    'id'               => 0
+	, 'instance_name'    => $aOptions['instance-name']
+	, 'start_ts'         => time()
+	, 'stop_ts'          => 0
+	, 'rc'               => 9999
+	, 'backup_type_id'   => getBackupTypeId($aOptions['type'])
+	, 'backup_policy_id' => getBackupPolicyId($aOptions['policy'])
+	, 'backup_mode_id'   => getBackupModeId($aOptions['mode'])
+	, 'compressed'       => isset($aOptions['no-compress']) ? 0 : 1
+	, 'archived'         => isset($aOptions['archive']) ? 1 : 0
+	, 'cleanedup'        => isset($aOptions['cleanup']) ? 1 : 0
+	, 'version'          => $gRelease
+	, 'hostname'         => $gHostname
+	, 'schema_name'      => ''
+	, 'binlog_file'      => ''
+	, 'binlog_pos'       => 0
+	, 'command'          => ''
+	, 'log'              => ''
+	, 'mysqli'           => $mysqli
+  );
+  
+	list($rc, $id) = createBackupInCatalog($mysqli, $aBackupInformation);
+	$aBackupInformation['id'] = $id;
+}
+
+tee("\nStart " . $aOptions['type'] . ' ' . $aOptions['policy'] . " backup at $gBackupTimestamp");
+
+// Do backups
+
 if ( $aOptions['type'] == 'full' ) {
   if ( $aOptions['mode'] == 'logical' ) {
-    $rc = doFullLogicalBackup($aTarget, $aOptions);
+    $rc = doFullLogicalBackup($aTarget, $aOptions, $aBackupInformation);
   }
   elseif ( $aOptions['mode'] == 'physical' ) {
-    $rc = doFullPhysicalBackup($aTarget, $aOptions);
+    $rc = doFullPhysicalBackup($aTarget, $aOptions, $aBackupInformation);
   }
 }
 elseif ( $aOptions['type'] == 'structure' ) {
-  $rc = doStructureBackup($aTarget, $aOptions);
+  $rc = doStructureBackup($aTarget, $aOptions, $aBackupInformation);
 }
 elseif ( $aOptions['type'] == 'binlog' ) {
-  $rc = doBinlogBackup($aTarget, $aOptions);
+  $rc = doBinlogBackup($aTarget, $aOptions, $aBackupInformation);
 }
 elseif ( $aOptions['type'] == 'config' ) {
-  $rc = doConfigBackup($aTarget, $aOptions);
+  $rc = doConfigBackup($aTarget, $aOptions, $aBackupInformation);
 }
 elseif ( $aOptions['type'] == 'cleanup' ) {
-  $rc = doCleanup($aTarget, $aOptions);
+  $rc = doCleanup($aOptions, $aBackupInformation);
 }
 elseif ( $aOptions['type'] == 'schema' ) {
-  $rc = doSchemaBackup($aTarget, $aOptions);
+  $rc = doSchemaBackup($aTarget, $aOptions, $aBackupInformation);
+}
+elseif ( $aOptions['type'] == 'privilege' ) {
+  $rc = doPrivilegeBackup($aTarget, $aOptions, $aBackupInformation);
+}
+elseif ( $aOptions['type'] == 'catalog' ) {
+
+	if ( $aOptions['policy'] == 'create' ) {
+		$rc = createCatalog($mysqli, $aOptions['catalog-name']);
+		if ( $rc == OK ) {
+			tee("  Catalog successfully created.");
+		}
+	}
+	elseif ( $aOptions['policy'] == 'upgrade' ) {
+		$rc = upgradeCatalog($mysqli, $aOptions['catalog-name']);
+		if ( $rc == OK ) {
+			tee("  Catalog successfully upgraded.");
+		}
+	}
+	else {
+		$rc = 197;
+		tee("  ERROR: Unexpected error. This should never happen (type=" . $aOptions['type'] . ', policy=' . $aOptions['policy'] . ")! Please report a bug! (rc=$rc).");
+	}
 }
 else {
-  $rc = 163;
-  tee("  ERROR: Unexpected error. This should never happen (" . $aOptions['type'] . ")! Please report a bug! (rc=$rc).");
-  exit($rc);
+	$rc = 163;
+	tee("  ERROR: Unexpected error. This should never happen (" . $aOptions['type'] . ")! Please report a bug! (rc=$rc).");
 }
 
 $lEndTs = date("Y-m-d_H-i-s");
+
+// Write backup start tag into catalog
+if ( array_key_exists('catalog', $aOptions) && (! array_key_exists('create', $aOptions)) && (! array_key_exists('upgrade', $aOptions)) ) {
+	$aBackupInformation['stop_ts'] = time();
+	$aBackupInformation['rc']      = $rc;
+	list($ret, $cnt) = setBackupInCatalog($mysqli, $aBackupInformation);
+	list($ret, $cnt) = setBackupDetailsInCatalog($mysqli, $aBackupInformation);
+}
+
+if ( array_key_exists('catalog', $aOptions) ) {
+	$ret = closeCatalog($mysqli);
+}
+
 tee("End " . $aOptions['type'] . " " . $aOptions['policy'] . " backup at $lEndTs (rc=$rc)");
 exit($rc);
 
